#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_utils.decorator module

This module only provides a single decorator, which can be used to mark a function as
deprecated.
"""

__docformat__ = 'restructuredtext'

import functools
import warnings


def deprecated(*msg):
    """This is a decorator which can be used to mark functions as deprecated.

    It will result in a warning being emitted when the function is used.
    """

    def decorator(func):

        @functools.wraps(func)
        def new_func(*args, **kwargs):
            warnings.warn_explicit("Function %s is deprecated. %s" % (func.__name__, message),
                                   category=DeprecationWarning,
                                   filename=func.__code__.co_filename,
                                   lineno=func.__code__.co_firstlineno + 1)
            return func(*args, **kwargs)
        return new_func

    if len(msg) == 1 and callable(msg[0]):
        message = u''
        return decorator(msg[0])
    else:
        message = msg[0]
        return decorator
